#include "mclib.h"
#include "drawpd.h"
#include <fstream.h>
#include "parse.h"
#include "getvalue.h"
#include "lstsqr.h"
#include <ctype.h>
#include <strstream.h>
#include "version.h"


int scan_mu_ordered_phase(Real *min_mu, Array<Real> *xs, Array<Real> *Es,
                          Real *mu_0, Real T,
                          Real max_dmu, Real mu_step, int dontdodir,
                          LinkedList<Real> *prev_order,
                          MonteCarlo *pmc, const Structure &str,
                          int n_equil, int n_step, Real threshold) {
  LinkedList<Real> x_list;
  LinkedList<Real> E_list;
  *min_mu=*mu_0;

  pmc->init_structure(str);
  pmc->init_run(T,*mu_0);
  pmc->run(n_equil);
  pmc->init_run();
  pmc->run(n_step);
  Real x=pmc->get_mean_concentration();
  Real E=pmc->get_mean_energy();
  Real mid_order=pmc->get_mean_disorder_param();
  Array<Real> prev_order_array;
  LinkedList_to_Array(&prev_order_array,*prev_order);
  for (int ii=0; ii<prev_order_array.get_size(); ii++) {cerr << prev_order_array(ii) << " "; cerr << endl << mid_order << endl;}
  if (detect_discontinuity(prev_order_array,mid_order)>threshold) {
    xs->resize(0);
    Es->resize(0);
    return 0;
  }
  x_list << new Real(x);
  E_list << new Real(E);
  
  Real last_valid_order=mid_order;
  for (int dir=-1; dir<=1; dir+=2) {
    if (dir==dontdodir) continue;
    LinkedList<Real> order_list;
    order_list << new Real(mid_order);
    pmc->init_structure(str);
    Real dmu=mu_step*(Real)dir;
    Real mu;
    for (mu=(*mu_0)+dmu; fabs(mu-*mu_0)<=max_dmu; mu+=dmu) {
      pmc->init_run(T,mu);
      pmc->run(n_equil);
      pmc->init_run();
      pmc->run(n_step);
      Real order=pmc->get_mean_disorder_param();
cerr << T << " " << mu << " " << order << endl;
      if (order_list.get_size()>1) {
        Array<Real> order_array;
        LinkedList_to_Array(&order_array,order_list);
        if (detect_discontinuity(order_array,order)>threshold) break;
      }
      else {
        if (detect_discontinuity(prev_order_array,order)>threshold) break;
      }
      Real x=pmc->get_mean_concentration();
      Real E=pmc->get_mean_energy();
      if (dir==-1) {
        *min_mu=mu;
        x_list.push_front(new Real(x));
        E_list.push_front(new Real(E));
      }
      else {
        x_list << new Real(x);
        E_list << new Real(E);
      }
      last_valid_order=order;
      order_list << new Real(order);
    }
    if (fabs(mu-*mu_0)>max_dmu) return 0;
  }
  LinkedList_to_Array(xs,x_list);
  LinkedList_to_Array(Es,E_list);
  if (dontdodir==0) {
    int mid=xs->get_size()/2;
    *mu_0=*min_mu+(Real)mid*mu_step;
  }
  if (dontdodir==0) {
    (*prev_order) << new Real(mid_order);
  }
  else {
    (*prev_order) << new Real(last_valid_order);
  }
  return 1;
}

void scan_ordered_phase(Array<ArrayReal> *phis, Array<Real> *min_mus, Array<ArrayReal> *xs,
                          Real phi_0, Real mu_0, Real T_0,
                          Real max_dmu, Real mu_step, int dontdodir, Real T_step,
                          MonteCarlo *pmc, const Structure &str,
                          int n_equil, int n_step, Real threshold) {
  LinkedList<Real> min_mu_list;
  LinkedList<ArrayReal> x_list;
  LinkedList<ArrayReal> E_list;
  Real T=T_0;
  Real mu=mu_0;
  Array<ArrayReal> Es;
  LinkedList<Real> prev_order;

  while (1) {
    Real min_mu;
    Array<Real> *pxs=new Array<Real>;
    Array<Real> *pEs=new Array<Real>;
    if (!scan_mu_ordered_phase(&min_mu,pxs,pEs,&mu, T,max_dmu,mu_step,dontdodir,&prev_order,
      pmc,str, n_equil,n_step, threshold)) break;
    min_mu_list << new Real(min_mu);
    x_list << pxs;
    E_list << pEs;
    T+=T_step;
  }
  LinkedList_to_Array(min_mus,min_mu_list);
  LinkedList_to_Array(xs,x_list);
  LinkedList_to_Array(&Es,E_list);
  phis->resize(xs->get_size());
  T=T_0;
  mu=mu_0;
  Real phi=phi_0;
  for (int i=0; i<phis->get_size(); i++) {
    (*phis)(i).resize((*xs)(i).get_size());
    if (i>0) {
      mu=( MAX((*min_mus)(i),(*min_mus)(i-1)) + MIN((*min_mus)(i)+mu_step*((*xs)(i).get_size()-1),(*min_mus)(i-1)+mu_step*((*xs)(i-1).get_size()-1)) )/2;
      int j1=(int)floor((mu-(*min_mus)(i))/mu_step);
      mu=(*min_mus)(i)+mu_step*(Real)j1;
      int j0=(int)floor((mu-(*min_mus)(i-1))/mu_step);
      Real E=(Es(i-1)(j0)+Es(i)(j1))/2.;
      Real b0=1./(T-T_step);
      Real b1=1./T;
      phi=((*phis)(i-1)(j0)*b0+E*(b1-b0))/b1;
    }
    (*phis)(i)(0)=0.;
    for (int j=1; j<(*phis)(i).get_size(); j++) {
      (*phis)(i)(j)=(*phis)(i)(j-1)-((*xs)(i)(j-1)+(*xs)(i)(j))/2*mu_step;
    } 
    int j0=(int)floor((mu-(*min_mus)(i))/mu_step);
    Real phiold=(*phis)(i)(j0);
    for (int j=0; j<(*phis)(i).get_size(); j++) {
      (*phis)(i)(j)=(*phis)(i)(j)-phiold+phi;
    }
  }
}

void find_mu_limit_disordered(Real *mu_l, Real *mu_r, Real *mu0, Real mu_step,
                              SampledThermoData *lphase, SampledThermoData *rphase) {
  const Array<ArrayReal> &l_xs=lphase->get_x_data();
  const Array<ArrayReal> &r_xs=rphase->get_x_data();
  int l_top=l_xs.get_size()-1;
  int r_top=r_xs.get_size()-1;
  if (mu_l) *mu_l=lphase->get_mu_data()(l_top)+(l_xs(l_top).get_size()-1)*mu_step/2.-mu_step;
  if (mu_r) *mu_r=rphase->get_mu_data()(r_top)+(r_xs(r_top).get_size()-1)*mu_step/2.+mu_step;
  if (mu0) {
    Array<Real> r_min_mus(rphase->get_mu_data());
    Array<Real> l_max_mus(lphase->get_mu_data());
    for (int i=0; i<l_max_mus.get_size(); i++) {
      l_max_mus(i)+=mu_step*(l_xs(i).get_size()-1);
    }
    for (int i=0; i<MIN(l_xs.get_size(),r_xs.get_size())-1; i++) {
      if (l_max_mus(i)>=r_min_mus(i)) {
        l_max_mus(i)=-MAXFLOAT;
        r_min_mus(i)= MAXFLOAT;
      }
    }
    *mu0=(max(l_max_mus)+min(r_min_mus))/2.;
  }
}

void scan_disordered_phase(Array<ArrayReal> *phis, Array<Real> *min_mus,
                   Array<ArrayReal> *xs, Real phi_0, Real T_0, Real T_min,
                         Real mu_step, Real T_step,
                   SampledThermoData *lphase, SampledThermoData *rphase,
                         MonteCarlo *pmc, int n_equil, int n_step) {
  Real mu_l,mu_r,mu_0;
  find_mu_limit_disordered(&mu_l,&mu_r,&mu_0,mu_step,lphase,rphase);

  LinkedList<Real> min_mu_list;
  LinkedList<ArrayReal> x_listlist;
  LinkedList<ArrayReal> phi_listlist;

  Real phi_mid=phi_0;
  Real prev_E=0;
  for (Real T=T_0; T>=T_min; T-=T_step) {
    Real min_mu=mu_0;
    pmc->init_random();
    pmc->init_run(T,mu_0);
    pmc->run(n_equil);
    pmc->init_run();
    pmc->run(n_step);
    Real x=pmc->get_mean_concentration();
    Real E=pmc->get_mean_energy();
    if (T!=T_0) {
      Real b0=1./(T+T_step);
      Real b1=1./T;
      phi_mid=(phi_mid*b0+((E+prev_E)/2.)*(b1-b0))/b1;
    }
    prev_E=E;
    if (phi_mid>lphase->phi(T,mu_0) || phi_mid>rphase->phi(T,mu_0)) break;

    LinkedList<Real> x_list;
    LinkedList<Real> phi_list;
    x_list << new Real(x);
    phi_list << new Real(phi_mid);

    Real phi=phi_mid;
    Real prev_x=x;
    for (int dir=-1; dir<=1; dir+=2) {
      pmc->init_random();
      Real dmu=mu_step*(Real)dir;
      Real mu;
      for (mu=mu_0+dmu; mu>=mu_l && mu<=mu_r; mu+=dmu) {
        pmc->init_run(T,mu);
        pmc->run(n_equil);
        pmc->init_run();
        pmc->run(n_step);
        Real x=pmc->get_mean_concentration();
        phi+=-(x+prev_x)*mu_step/2.;
	cout << T << " " << mu << " " << x << " " << phi << " " << lphase->phi(T,mu) << " " << rphase->phi(T,mu) << endl;
        prev_x=x;
        if (dir==-1) {
          min_mu=mu;
          x_list.push_front(new Real(x));
          phi_list.push_front(new Real(phi));
	  if (phi>lphase->phi(T,mu)) break;
        }
        else {
          x_list << new Real(x);
          phi_list << new Real(phi);
	  if (phi>rphase->phi(T,mu)) break;
        }
	if (phi>lphase->phi(T,mu) || phi>rphase->phi(T,mu)) break;
      }
    }
    min_mu_list << new Real(min_mu);
    Array<Real> *pxs=new Array<Real>;
    Array<Real> *pphis=new Array<Real>;
    LinkedList_to_Array(pxs,x_list);
    LinkedList_to_Array(pphis,phi_list);
    x_listlist << pxs;
    phi_listlist << pphis;
  }
  LinkedList_to_Array(min_mus,min_mu_list);
  LinkedList_to_Array(xs,x_listlist);
  LinkedList_to_Array(phis,phi_listlist);
}

Real calc_disordered_phi(Real b_0, Real b_step, Real mu,
                   PhaseThermoData *hte,
                         MonteCarlo *pmc, int n_equil, int n_step, Real phi_prec, Real *b_high=NULL) {
  Real prev_E=0;
  Real prev_phi=0;
  Real b;
  Real phi;
  Real db_phi=0;
  for (b=b_0; 1; b-=b_step) {
    pmc->init_random();
    pmc->init_run(1/b,mu);
    pmc->run(n_equil);
    pmc->init_run();
    pmc->run(n_step);
    Real E=pmc->get_mean_energy();
    Real b_phi=b*(hte->phi(1/b,mu));
    if (b!=b_0) {
      db_phi+=((E+prev_E)/2.)*b_step;
      phi=(b_phi-db_phi)/b_0;
      if (fabs(phi-prev_phi)*b/b_step<phi_prec) break;
    }
    else {
      phi=b_phi/b_0;
    }
    if (b<3*b_step/2) break;
    cout << E << " " << b_phi << " " << phi << " " << db_phi << endl;
    prev_E=E;
    prev_phi=phi;
  }
  if (b_high) {*b_high=b;}
  return phi;
}

int main(int argc, char *argv[]) {
  Real enclosed_radius=0.;
  Real mu_step=0.05;
  Real T_step=0.1;
  Real x_step=1e-3;
  Real T_max=MAXFLOAT;
  Real phi_prec=0.01;
  int n_equil=200;
  int n_step=500;
  Real threshold=3.;
  Real mu_edge_mult=2.;
  int read_in=0;
  int def_val=0;
  int do_subrun=0;
  AskStruct options[]={
    {"","AUTO Monte Carlo, by Axel van de Walle",TITLEVAL,NULL},
    {"-er","enclosed radius",REALVAL,&enclosed_radius},
    {"-dmu","mu step",REALVAL,&mu_step},
    {"-dT","T step",REALVAL,&T_step},
    {"-dx","x step",REALVAL,&x_step},
    {"-Tmax","Maximum T",REALVAL, &T_max},
    {"-dphi","Free energy precision",REALVAL,&phi_prec},
    {"-eq","n equil",INTVAL,&n_equil},
    {"-n","n step",INTVAL,&n_step},
    {"-tstat","Critical value of the test for discontinuity",REALVAL,&threshold},
    {"-r","Read data from mc.out",BOOLVAL,&read_in},
    {"-p","which piece of the phase diagram to calculate (from 1 to 2*nb_gs-1)",INTVAL,&do_subrun},
    {"-d","Use default values",BOOLVAL,&def_val}
  };
  if (!get_values(argc,argv,countof(options),options)) {
    display_help(countof(options),options);
    return 1;
  }

  if (read_in) {
    LinkedList<PhaseThermoData> phase_list;
    PhaseThermoData *phase;
    {
      ifstream file("mc.out");
      if (!file) ERRORQUIT("Unable to open mc.out");
      while (phase=read_phase(file)) {phase_list << phase;}
    }
    Array<Real> mu_hints;
    {
      LinkedList<Real> mu_list;
      ifstream file("mu_hint.out");
      while (1) {
        Real mu=MAXFLOAT;
        file >> mu;
        if (mu==MAXFLOAT) break;
        mu_list << new Real(mu);
      }
      LinkedList_to_Array(&mu_hints,mu_list);
    }
    LinkedList<TwoPhaseRegion> two_phase_list;
    calc_phase_boundaries(&two_phase_list,phase_list,T_step,x_step,mu_hints);
    {
      ofstream file("pd.out");
      write_phase_diagram(two_phase_list,file);
    }
    return 0;
  }

  Structure lattice;
  Array<Arrayint> labellookup;
  Array<AutoString> label;
  rMatrix3d axes;
  {
    ifstream latticefile("lat.in");
    if (!latticefile) ERRORQUIT("Unable to open lat.in");
    parse_lattice_file(&lattice.cell, &lattice.atom_pos, &lattice.atom_type, &labellookup, &label, latticefile, &axes);
    wrap_inside_cell(&lattice.atom_pos,lattice.atom_pos,lattice.cell);
  }
  for (int i=0; i<labellookup.get_size(); i++) {
    if (labellookup(i).get_size()>2)
      ERRORQUIT ("Sorry, only binary alloys implemented");
  }
  SpaceGroup spacegroup;
  spacegroup.cell=lattice.cell;
  find_spacegroup(&spacegroup.point_op,&spacegroup.trans,lattice.cell,lattice.atom_pos,lattice.atom_type);

  int nbsite=0;
  for (nbsite=0; nbsite<lattice.atom_pos.get_size(); nbsite++) {
    if (labellookup(lattice.atom_type(nbsite)).get_size()==1) break;
  }
  if (nbsite==0) ERRORQUIT("Need at least one site with multiple species in lattice file.");
  Structure lattice_only;
  lattice_only.cell=lattice.cell;
  lattice_only.atom_pos.resize(nbsite);
  lattice_only.atom_type.resize(nbsite);
  for (int i=0; i<lattice_only.atom_pos.get_size(); i++) {
    lattice_only.atom_pos(i) =lattice.atom_pos(i);
    lattice_only.atom_type(i)=lattice.atom_type(i);
  }

  LinkedList<Cluster> clusterlist;
  LinkedList<Real> ecilist;
  {
    ifstream clusterfile("clusters.out");
    if (!clusterfile) ERRORQUIT("Unable to open clusters.out");
    ifstream ecifile("eci.out");
    if (!ecifile) ERRORQUIT("Unable to open eci.out");
    Real maxlen=0.;
    while (1) {
      Real cur_eci;
      ecifile >> cur_eci;
      Real mult;
      clusterfile >> mult;
      if (clusterfile.eof()) break;
      Real len;
      clusterfile >> len;
      int nbpt;
      clusterfile >> nbpt;
      Cluster *pcluster=new Cluster(nbpt);
      for (int j=0; j<nbpt; j++) {
	clusterfile >> (*pcluster)(j);
	(*pcluster)(j)=axes*((*pcluster)(j));
      }
      clusterlist << pcluster;
      maxlen=MAX(maxlen,get_cluster_length(*pcluster));
      ecilist << new Real(cur_eci);
    }
    if (enclosed_radius==0) enclosed_radius=maxlen;
  }

  Array<Structure> gs_list;
  {
    ifstream strfile("gs_str.out");
    if (!strfile) ERRORQUIT("Unable to open gs_str.out");
    LinkedList<Structure> gs_llist;
    while (!strfile.eof()) {
      Structure str;
      parse_structure_file(&str.cell,&str.atom_pos,&str.atom_type,label,strfile);
      if (!fix_atom_type(&str, lattice, labellookup,1)) ERRORQUIT("Error reading gs_str.out");
      skip_to_next_structure(strfile);
      wrap_inside_cell(&str.atom_pos,str.atom_pos,str.cell);
      for (int s=0; s<str.atom_type.get_size(); s++) {
        str.atom_type(s)=-1+2*str.atom_type(s);
      }
      gs_llist << new Structure(str);
    }
    LinkedList_to_Array(&gs_list,gs_llist);
  }
  int nb_gs=gs_list.get_size();

  Array<Real> gs_concentration(nb_gs),gs_energy(nb_gs),gs_mid_mu(nb_gs);
  for (int gs=0; gs<nb_gs; gs++) {
    iVector3d simple_supercell;
    find_common_simple_supercell(&simple_supercell, lattice_only.cell,gs_list(gs).cell);
    MonteCarlo mc(lattice_only,simple_supercell,spacegroup,clusterlist,ecilist);
    mc.init_structure(gs_list(gs));
    gs_concentration(gs)=mc.get_cur_concentration();
    gs_energy(gs)=mc.get_cur_energy();
  }
  for (int gs=0; gs<nb_gs; gs++) {
    if (gs==0) {
      gs_mid_mu(gs)=(gs_energy(1)-gs_energy(0))/(gs_concentration(1)-gs_concentration(0));
    }
    else if (gs==nb_gs-1) {
      gs_mid_mu(gs)=(gs_energy(nb_gs-1)-gs_energy(nb_gs-2))/(gs_concentration(nb_gs-1)-gs_concentration(nb_gs-2));
    }
    else {
      gs_mid_mu(gs)=(gs_energy(gs+1)-gs_energy(gs-1))/(gs_concentration(gs+1)-gs_concentration(gs-1));
    }
  }
  Real mu_mid=(gs_energy(nb_gs-1)-gs_energy(0))/(gs_concentration(nb_gs-1)-gs_concentration(0));
  Real eci_strength;
  {
    LTE_ThermoData gs0(lattice_only,spacegroup,gs_list(0),clusterlist,ecilist);
    Structure disordered=lattice_only;
    for (int i=0; i<disordered.atom_type.get_size(); i++) {disordered.atom_type(i)=0;}
    LTE_ThermoData disord(lattice_only,spacegroup,disordered,clusterlist,ecilist);
    LTE_ThermoData gs1(lattice_only,spacegroup,gs_list(nb_gs-1),clusterlist,ecilist);
    eci_strength=fabs(disord.phi(0,0)-(gs0.phi(0,0)+gs1.phi(0,0))/2.);
  }
  gs_mid_mu(0)=(gs_energy(1)-gs_energy(0))/(gs_concentration(1)-gs_concentration(0))-mu_edge_mult*eci_strength;
  gs_mid_mu(nb_gs-1)=(gs_energy(nb_gs-1)-gs_energy(nb_gs-2))/(gs_concentration(nb_gs-1)-gs_concentration(nb_gs-2))+mu_edge_mult*eci_strength;

  Real max_dmu=gs_mid_mu(nb_gs-1)-gs_mid_mu(0);
  mu_step=mu_step*max_dmu;

  if (do_subrun==0) {
    ofstream file("mu_hint.out");
    for (int gs=0; gs<nb_gs; gs++) {
      file << gs_mid_mu(gs) << endl;
    }
  }

  // Calculations;
  {
    Real T_max_lte=MAXFLOAT; // highest T where LTE holds;
    for (int gs=0; gs<nb_gs; gs++) {
      LTE_ThermoData lte(lattice_only,spacegroup,gs_list(gs),clusterlist,ecilist);
      MF_ThermoData mf(lattice_only,spacegroup,gs_list(gs),clusterlist,ecilist);
      Real T;
      Real f=0.05;
      Real mu1=(f*gs_mid_mu(MAX(gs-1,0))+(1-f)*gs_mid_mu(gs));
      Real mu2=(f*gs_mid_mu(MIN(gs+1,nb_gs-1))+(1-f)*gs_mid_mu(gs))/2.;
      for (T=T_step/2; fabs(lte.phi(T,mu1)-mf.phi(T,mu1))<phi_prec && fabs(lte.phi(T,mu2)-mf.phi(T,mu2))<phi_prec; T+=T_step) {
       	cout << lte.phi(T,mu1) << " " << mf.phi(T,mu1) << " " << lte.phi(T,mu2) << " " << mf.phi(T,mu2) << endl;
      }
      T_max_lte=MIN(T,T_max_lte);
    }

    int sub_run=1;

    LinkedList<PhaseThermoData> list_phase;
    LinkedList<SampledThermoData> list_phase_mc;
    Array<Real> T_top(nb_gs);
    for (int gs=0; gs<nb_gs; gs++, sub_run++) {
      PhaseListT *phase=new PhaseListT();
      PhaseThermoData *lte=new LTE_ThermoData(lattice_only,spacegroup,gs_list(gs),clusterlist,ecilist);
      phase->add_phase(lte,T_max_lte);
      SampledThermoData *mc_data=NULL;
      char bufdumpname[MAX_LINE_LEN]; ostrstream dumpname(bufdumpname,MAX_LINE_LEN);
      dumpname << "dump" << sub_run << ".out" << '\0';
      ifstream dumpfilein(dumpname.str());
      if ((do_subrun!=sub_run || do_subrun==0) && dumpfilein) {
	mc_data=new SampledThermoData();
	mc_data->read(dumpfilein);
      }
      else {
	Array<ArrayReal> phis;
	Array<Real> min_mus;
	Array<ArrayReal> xs;
	iMatrix3d fit_sphere;
	fit_sphere.diag(find_sphere_bounding_box(gs_list(gs).cell,enclosed_radius));
	rMatrix3d supercell_fit=gs_list(gs).cell*to_real(fit_sphere);
	iVector3d simple_supercell;
	find_common_simple_supercell(&simple_supercell,lattice_only.cell,supercell_fit);
	MonteCarlo mc(lattice_only,simple_supercell,spacegroup,clusterlist,ecilist);
	Real phi0=lte->phi(T_max_lte,gs_mid_mu(gs));
	int dont_do_dir=(gs==0 ? -1 : ( (gs==nb_gs-1) ? 1 : 0 ) );
	Real cur_max_dmu=(gs==0 ? gs_mid_mu(1)-gs_mid_mu(0) : (gs==nb_gs-1 ? gs_mid_mu(nb_gs-1)-gs_mid_mu(nb_gs-2) : max_dmu) );
	scan_ordered_phase(&phis,&min_mus,&xs,
			   phi0, gs_mid_mu(gs),T_max_lte,cur_max_dmu,mu_step,
			   dont_do_dir,
			   T_step,&mc,gs_list(gs),n_equil,n_step,threshold);
	mc_data=new SampledThermoData(T_max_lte,T_step,min_mus,mu_step,xs,phis);
	ofstream dumpfile(dumpname.str());
	mc_data->write(dumpfile);
      }
      T_top(gs)=T_max_lte+(Real)(mc_data->get_mu_data().get_size()-1)*T_step;
      phase->add_phase(mc_data,T_top(gs));
      list_phase << phase;
      list_phase_mc << mc_data;
    }

    Real max_T_top=max(T_top); // largest disordering T;

    PhaseThermoData *hte=new HTE_ThermoData(lattice_only.atom_pos.get_size(),spacegroup,clusterlist,ecilist);
    Real T_min_hte; // lowest T where HTE holds;
    iVector3d simple_supercell=find_sphere_bounding_box(lattice.cell,enclosed_radius);
    MonteCarlo rnd_mc(lattice_only,simple_supercell,spacegroup,clusterlist,ecilist);
    calc_disordered_phi(1./max_T_top, 1./max_T_top-1./(max_T_top+T_step), mu_mid,
                   hte,&rnd_mc,n_equil,n_step,phi_prec,&T_min_hte);
    T_min_hte=1./T_min_hte;
    if (T_max==0) {T_max=max_T_top+T_step;}
    Real T_top_mc; // highest T at which MonteCarlo is performed;
    T_top_mc=MIN(T_max,T_min_hte);
    PhaseList *rnd_middle=new PhaseList();
    LinkedListIterator<SampledThermoData> p(list_phase_mc);
    LinkedListIterator<SampledThermoData> p_next(list_phase_mc);
    p_next++;
    for (int gs=0; gs<nb_gs-1; gs++,p++,p_next++,sub_run++) {
      SampledThermoData *piece_rnd=NULL;
      char bufdumpname[MAX_LINE_LEN]; ostrstream dumpname(bufdumpname,MAX_LINE_LEN);
      dumpname << "dump" << sub_run << ".out" << '\0';
      ifstream dumpfilein(dumpname.str());
      if ((do_subrun!=sub_run || do_subrun==0) && dumpfilein) {
	piece_rnd=new SampledThermoData();
	piece_rnd->read(dumpfilein);
      }
      else {
	SampledThermoData *lphase=p;
	SampledThermoData *rphase=p_next;
	Array<Real> mus(1);
	mus(0)=gs_mid_mu(0);
	Array<ArrayReal> tmp(1);
	SampledThermoData dummy(T_max_lte,T_step,mus,mu_step,tmp,tmp);
	if (gs==0) {lphase=&dummy;}
	else if (gs==nb_gs-1) {rphase=&dummy;}
	Real mu_0;
	find_mu_limit_disordered(NULL,NULL,&mu_0,mu_step,lphase,rphase);
	Real phi_0=calc_disordered_phi(1./T_top_mc, 1./max_T_top-1./(max_T_top+T_step), mu_0,
				       hte,&rnd_mc,n_equil,n_step,phi_prec);
	Array<ArrayReal> phis;
	Array<Real> min_mus;
	Array<ArrayReal> xs;
	Real T_min=((gs==0 || gs==nb_gs-1) ? T_top(gs)-2*T_step : 0.);
	scan_disordered_phase(&phis,&min_mus,&xs,phi_0,T_top_mc,T_min,mu_step,T_step,
			      lphase,rphase,&rnd_mc,n_equil,n_step);
	piece_rnd=new SampledThermoData(T_top_mc,T_step,min_mus,mu_step,xs,phis);
      }
      rnd_middle->add_phase(piece_rnd);
      ofstream dumpfile(dumpname.str());
      piece_rnd->write(dumpfile);
    }
    {
      PhaseListT *rnd_top=new PhaseListT();
      rnd_top->add_phase(rnd_middle,T_top_mc);
      rnd_top->add_phase(hte);
      LinkedListIterator<PhaseThermoData> p(list_phase);
      PhaseThermoData *rnd_left=list_phase.detach(p);
      for (int i=0; i<nb_gs-2; i++,p++) {};
      PhaseThermoData *rnd_right=list_phase.detach(p);
      PhaseThermoData *rnd_phase=new MiscibilityGap(rnd_top,rnd_left,rnd_right);
      list_phase << rnd_phase;
      list_phase_mc.detach_all();
    }
    if (do_subrun==0) {
      ofstream file("mc.out");
      LinkedListIterator<PhaseThermoData> p(list_phase);
      for ( ; p; p++) {
        p->write(file);
      }
    }
  }
}
