#include "mmclib.h"
//#include "drawpd.h"
#include <fstream.h>
#include <strstream.h>
#include "parse.h"
#include "getvalue.h"
#include "lstsqr.h"
#include "version.h"
#include "teci.h"
#include "equil.h"
//#include "keci.h"
//#include "plugin.h"

//extern 
char *helpstring="";

//GenericPlugIn<KSpaceECI> *GenericPlugIn<KSpaceECI>::list=NULL;

void print_vector(ostream &file, const Array<Real> &v, char *delim) {
  for (int i=0; i<v.get_size(); i++) {
    file << v(i) << delim;
  }
}

int main(int argc, char *argv[]) {
  char *delim="\t";
  int help=0;
  Real enclosed_radius=0.;
  int n_equil=-1;
  int n_step=-1;
  Real t_prec=0.;
  int which_col=0;
  int init_gs=-2;
  Real threshold=3.;
  int sigdig=6;
  int quiet=0;
  char *outfile="mc.out";
  Real kboltzman=1.;
  int keV=0;
  int seed=0;
  int droplast=0;
  int addmux=0;
  char *my_init_str="";
  //  char *kspace_labels="";
  AskStruct options[]={
    {"","Eazy Monte Carlo Code " MAPS_VERSION ", by Axel van de Walle",TITLEVAL,NULL},
    {"-h","Help",BOOLVAL,&help},
    {"-er","set the system  size so that a sphere of that radius must fit inside the simulation cell",REALVAL,&enclosed_radius},
    {"-eq","number of equilibration passes",INTVAL,&n_equil},
    {"-n","number of averaging passes",INTVAL,&n_step},
    {"-tp","Target precision (optional, replaces -n and -eq)",REALVAL,&t_prec},
    {"-aq","Quantity that must meet the tolerance specified by -tp. "
           "0: energy (default), 1: long-range order, 2-: concentation, correlations",INTVAL,&which_col},
    {"-gs","which ground state to use as initial config (-gs=-1 to use random state)",INTVAL,&init_gs},
    {"-tstat","Critical value of the test for discontinuity",REALVAL,&threshold},
    {"-sigdig","Number of significant digits printed",INTVAL,&sigdig},
    {"-q","Quiet (do not write to stdout)",BOOLVAL,&quiet},
    {"-o","Output file (default: mc.out)",STRINGVAL,&outfile},
    {"-k","Boltzman's constant (conversion factor from T to energy)",REALVAL,&kboltzman},
    {"-keV","Set Boltzman's constant to 8.617e-5 so that temperature is in K when energy is in eV",BOOLVAL,&keV},
    {"-sd","Seed for random number generation (default: use clock)",INTVAL,&seed},
    {"-dl","Drop the last data point of each inner loop (after the phase transition occured)",BOOLVAL,&droplast},
    {"-g2c","Convert output to canonical rather than grand-canonical quantities",BOOLVAL,&addmux},
    {"-is","User specified initial configuration (replaces -gs)",STRINGVAL,&my_init_str}
  };
  if (!get_values(argc,argv,countof(options),options)) {
    display_help(countof(options),options);
    return 1;
  }
  if (help) {
    cout << helpstring;
    exit(1);
  }
  if (strlen(my_init_str)>0) {
    init_gs=-1;
  }

  if (keV) {kboltzman=8.617e-5;}

  if (init_gs==-2)       ERRORQUIT("Specify -gs");
  if (! ((n_equil!=-1 && n_step!=-1) || t_prec>0.)) ERRORQUIT("Specify either (-eq and -n) or -dx");
  if (t_prec>0. && n_equil==-1) {n_equil=0;}
  if ( t_prec<0. ) ERRORQUIT("-dx value must be positive");

  rndseed(seed);

  Structure lattice;
  Array<Arrayint> labellookup;
  Array<AutoString> label;
  rMatrix3d axes;
  {
    ifstream latticefile("lat.in");
    if (!latticefile) ERRORQUIT("Unable to open lat.in");
    parse_lattice_file(&lattice.cell, &lattice.atom_pos, &lattice.atom_type, &labellookup, &label, latticefile, &axes);
    wrap_inside_cell(&lattice.atom_pos,lattice.atom_pos,lattice.cell);
  }

  TrigoCorrFuncTable corrfunc;
  corrfunc.init_from_site_type_list(labellookup);

  cerr << "Species:" << endl;
  for (int i=0; i<label.get_size(); i++) {
    cerr << label(i) << " " << endl;
  }
  Array<Real> control_org;
  Array<Real> maxdiv;
  Array2d<Real> dcontrol;
  {
    LinkedList<Array<Real> > dcontrol_list;
    LinkedList<Real> maxdiv_list;
    ifstream controlfile("control.in");
    if (!controlfile) {
      ERRORQUIT("control.in file not found.");
    }
    while (skip_delim(controlfile)) {
      Array<Real> cur_cont(label.get_size()+1);
      for (int i=0; i<cur_cont.get_size(); i++) {
	controlfile >> cur_cont(i);
      }
      if (control_org.get_size()==0) {
	control_org=cur_cont;
      }
      else {
	Real curmaxdiv;
	controlfile >> curmaxdiv;
	maxdiv_list << new Real(curmaxdiv);
	diff(&cur_cont,cur_cont,control_org);
	product(&cur_cont,cur_cont,1./curmaxdiv);
	dcontrol_list << new Array<Real>(cur_cont);
      }
    }
    LinkedList_to_Array(&maxdiv,maxdiv_list);
    dcontrol.resize(control_org.get_size(),dcontrol_list.get_size());
    LinkedListIterator<Array<Real> > idc(dcontrol_list);
    for (int i=0; idc; i++,idc++) {
      for (int j=0; j<control_org.get_size(); j++) {
	dcontrol(j,i)=(*idc)(j);
      }
    }
  }

  SpaceGroup spacegroup;
  spacegroup.cell=lattice.cell;
  find_spacegroup(&spacegroup.point_op,&spacegroup.trans,lattice.cell,lattice.atom_pos,lattice.atom_type);
  if (contains_pure_translations(spacegroup.point_op,spacegroup.trans)) {
    cerr << "Warning: unit cell is not primitive." << endl;
  }

  Array2d<Real> concmat;
  Array2d<Real> corr_to_conc;
  Array2d<Real> conc_to_fullconc;
  Array2d<Real> corr_to_fullconc;
  Array<Real> conc_to_fullconc_c;
  calc_corr_to_conc(&concmat,lattice,labellookup,spacegroup,corrfunc);
  extract_nonredundant(&corr_to_conc,&conc_to_fullconc,&conc_to_fullconc_c, concmat);
  product(&corr_to_fullconc,conc_to_fullconc,corr_to_conc);

  int nbsite=0;
  for (nbsite=0; nbsite<lattice.atom_pos.get_size(); nbsite++) {
    if (labellookup(lattice.atom_type(nbsite)).get_size()==1) break;
  }
  if (nbsite==0) ERRORQUIT("Need at least one site with multiple species in lattice file.");
  Structure lattice_only;
  lattice_only.cell=lattice.cell;
  lattice_only.atom_pos.resize(nbsite);
  lattice_only.atom_type.resize(nbsite);
  for (int i=0; i<lattice_only.atom_pos.get_size(); i++) {
    lattice_only.atom_pos(i) =lattice.atom_pos(i);
    lattice_only.atom_type(i)=lattice.atom_type(i);
  }

  LinkedList<MultiCluster> clusterlist;
  {
    ifstream clusterfile("clusters.out");
    if (!clusterfile) ERRORQUIT("Unable to open clusters.out");
    Real maxlen=read_clusters_and_eci(&clusterlist,NULL,clusterfile,clusterfile,axes);
    if (enclosed_radius==0) enclosed_radius=maxlen;
  }

  PolyInterpolatorBig<Array<Real> > teci;
  PolyInterpolatorBig<Array<Real> > teeci;
  read_t_dep_eci_file(&teci,&teeci,clusterlist.get_size(),kboltzman);

  Array<Structure> gs_list;
  {
    ifstream strfile("gs_str.out");
    if (!strfile) ERRORQUIT("Unable to open gs_str.out");
    LinkedList<Structure> gs_llist;
    while (!strfile.eof()) {
      Structure str;
      parse_structure_file(&str.cell,&str.atom_pos,&str.atom_type,label,strfile);
      if (!fix_atom_type(&str, lattice, labellookup,1)) ERRORQUIT("Error reading gs_str.out");
      skip_to_next_structure(strfile);
      wrap_inside_cell(&str.atom_pos,str.atom_pos,str.cell);
      gs_llist << new Structure(str);
    }
    LinkedList_to_Array(&gs_list,gs_llist);
  }
  int nb_gs=gs_list.get_size();
  if (init_gs>=nb_gs || init_gs<-1) ERRORQUIT("-gs option is out of range.");

  Structure disordered=lattice_only;
  for (int i=0; i<disordered.atom_type.get_size(); i++) {
    disordered.atom_type(i)=0;
  }

  Structure *p_init_str,*my_p_init_str;
  p_init_str=(init_gs==-1 ? &disordered : &gs_list(init_gs));
  Structure mystr;
  if (strlen(my_init_str)>0) {
    ifstream strfile(my_init_str);
    if (!strfile) ERRORQUIT("Unable to open initial structure file");
    parse_structure_file(&mystr.cell,&mystr.atom_pos,&mystr.atom_type,label,strfile);
    if (!fix_atom_type(&mystr, lattice, labellookup,1)) ERRORQUIT("Error reading initial structure file");
    wrap_inside_cell(&mystr.atom_pos,mystr.atom_pos,mystr.cell);
    my_p_init_str=&mystr;
  }
  else {
    my_p_init_str=p_init_str;
  }

  iVector3d simple_supercell;
  find_common_simple_supercell(&simple_supercell,lattice_only.cell,my_p_init_str->cell);
  rMatrix3d mat_simple_supercell;
  mat_simple_supercell.diag(to_real(simple_supercell));
  iVector3d fit_sphere;
  fit_sphere=find_sphere_bounding_box(lattice_only.cell*mat_simple_supercell,2.*enclosed_radius);
  for (int i=0; i<3; i++) {
    simple_supercell(i)*=fit_sphere(i);
  }
  if (!quiet) cout << "Supercell size: " << simple_supercell << endl;

  MultiMonteCarlo *pmc=new MultiMonteCarlo(lattice_only,labellookup,simple_supercell,spacegroup,clusterlist,corrfunc);

  Array<Real> mult;
  pmc->get_cluster_mult(&mult);

  ofstream mcfile(outfile);
  mcfile.setf(ios::fixed);
  mcfile.precision(sigdig);
  cout.setf(ios::fixed);
  cout.precision(sigdig);

  MultiDimIterator<Array<Real> > curpt(maxdiv);
  for (; curpt; curpt++) {
    if (((Array<Real> &)curpt)(0)==0) {
      pmc->init_structure(*my_p_init_str);
    }

    Array<Real> cur_control;
    product(&cur_control,dcontrol,curpt);
    sum(&cur_control,cur_control,control_org);
    Real T=cur_control(0)*kboltzman;
    Array<Real> full_mu;
    extract_elements(&full_mu,cur_control,1,cur_control.get_size());
    Array<Real> mu;
    product(&mu,full_mu,corr_to_fullconc);
    
    Array<Real> eci;
    teci.interpol(&eci,T);
    pmc->set_eci(eci);
    pmc->set_T_mu(T,mu);

    if (((Array<Real> &)curpt)(0)==0) {
      pmc->run(n_equil,1);
    }
    GenericAccumulator *paccum=create_accum(n_step,t_prec,which_col);
    run_mc(paccum,pmc,1);
    const Array<Real> &mcdata=paccum->get_mean();
    const Array<Real> &mcdata2=paccum->get_var();
    
    Real cur_E=mcdata(0)+inner_product(full_mu,conc_to_fullconc_c);
    Real cur_disorder=mcdata(1);
    Array<Real> corr,mult;
    extract_elements(&corr,mcdata,2+mu.get_size(),2+mu.get_size()+eci.get_size());
    pmc->get_cluster_mult(&mult);
    fix_energy(&cur_E, mult,corr,teci,T);

    Array<Real> full_conc,ptcorr;
    extract_elements(&ptcorr,mcdata,2,2+mu.get_size());
    product(&full_conc,corr_to_fullconc,ptcorr);
    sum(&full_conc,full_conc,conc_to_fullconc_c);
    Real mux=inner_product(full_mu,full_conc);

    ostrstream line;
    line << T/kboltzman << delim;
    print_vector(line,full_mu,delim);
    line << cur_E+(addmux ? mux : 0.) << delim;
    line << cur_disorder << delim;
    print_vector(line,full_conc,delim);
    print_vector(line,corr,delim);
    line << endl << '\0';
    mcfile << line.str() << flush;
    if (!quiet) {cout << line.str() << flush;}
    delete paccum;
  }
  {
    ofstream file("mcsnapshot.out");
    file.setf(ios::fixed);
    file.precision(sigdig);
    pmc->view(labellookup,label,file,axes);
  }
  delete pmc;
}

