#ifndef _MCLIB_H_
#define _MCLIB_H_

#include "clus_str.h"
#include "linalg.h"
#include <fstream.h>
#include "equil.h"
//#include <time.h>
//#include "keci.h"

#define SPIN_TYPE signed char

class MultiMonteCarlo {
 protected:
  Structure lattice;
Array<Array<int> > site_type_list;
  iVector3d supercell;
  iVector3d margin;
  iVector3d total_box;
  SPIN_TYPE *spin;
SPIN_TYPE *spin_orig;
  Real rspin_size;
  int site_in_cell;
int *nb_spin_val;
  int total_clusters;
  int *nb_clusters;
  //  int *nb_x_clusters;
  int **cluster_size;
  Real *rcluster_mult_per_atom;
  int ***site_offset;
Real ****spin_val_clus;
  Real **eci;
  int **which_cluster;
  Real E_ref;
  int which_is_empty;
int nb_point;
int *which_is_point;
Real *point_mult;
  Real cur_energy;
  Array<Real> cur_rho;
  Real *pcur_rho;
Real *new_rho;
  Real cur_disorder_param;
  Array<Real> cur_conc;

  Real T;
  Array<Real> mu;
  Real *pmu;
 public:
  void calc_from_scratch(void);
  void spin_flip(void);
  //  void spin_double_flip(void);
  void run(int mc_passes, int mode);
  MultiMonteCarlo(const Structure &_lattice, const Array<Array<int> > &_site_type_list, const iVector3d &_supercell, 
		const SpaceGroup &space_group, const LinkedList<MultiCluster> &cluster_list, 
		const Array<Array<Array<Real> > > &_corrfunc);
  ~MultiMonteCarlo(void);
  void set_eci(const Array<Real> &eci);
  void init_random(const Array<Array<Real> > &conc);
  void init_structure(const Structure &str);
  //  void set_concentration(Real concentration);
  void set_T_mu(Real _T, const Array<Real> &_mu);
  void view(const Array<Arrayint> &labellookup, const Array<AutoString> &atom_label, ofstream &file, const rMatrix3d &axes);

  const Array<Real> & get_cur_corr(void) const {
    return cur_rho;
  }
  Real get_cur_energy(void) const {
    return E_ref+cur_energy+extension_get_energy();
  }
  const Array<Real> & get_cur_concentration(void);
  Real get_cur_disorder_param(void) const {
    return cur_disorder_param;
  }
  int get_total_clusters(void) const {
    return total_clusters;
  }
  void get_cluster_mult(Array<Real> *mult) const {
    mult->resize(total_clusters);
    for (int i=0; i<total_clusters; i++) {(*mult)(i)=rcluster_mult_per_atom[i];}
  }
  Real get_T(void) const {return T;}
  const Array<Real> &get_mu(void) const {return mu;}
  const iVector3d & get_cell_size() const {
    return supercell;
  }
  void get_thermo_data(Array<Real> *pdata);

 protected:
  virtual void extension_calc_from_scratch(void) {}
  virtual void extension_update_spin_flip(int *cell, int incell, int newspin) {}
  virtual void extension_undo_spin_flip(void) {}
  virtual Real extension_spin_flip_energy(int *cell, int incell, int newspin) {return 0.;}
  virtual Real extension_get_energy(void) const {return 0.;}
};


void run_mc(GenericAccumulator *accum, MultiMonteCarlo *pmc, int mode);

#endif
