#include "parse.h"
#include "arraylist.h"
#include "calccorr.h"
#include "phonlib.h"
#include <fstream.h>

int main(int argc, char *argv[]) {
  char *latfilename="lat.in";
  char *strfilename="str.out";

  Structure lat;
  Array<Arrayint> site_type_list;
  Array<AutoString> atom_label;
  rMatrix3d axes;
  {
    ifstream file(latfilename);
    if (!file) ERRORQUIT("Unable to open lattice file.");
    parse_lattice_file(&lat.cell, &lat.atom_pos, &lat.atom_type, &site_type_list, &atom_label, file, &axes);
    if (fabs(det(lat.cell))<zero_tolerance) ERRORQUIT("Lattice vectors are coplanar.");
    wrap_inside_cell(&lat.atom_pos,lat.atom_pos,lat.cell);
  }
  SpaceGroup spacegroup;
  find_spacegroup(&spacegroup.point_op,&spacegroup.trans,lat.cell,lat.atom_pos,lat.atom_type);
  Array<rMatrix3d> pointgroup;
  pointgroup_from_spacegroup(&pointgroup, spacegroup.point_op);
  LinkedList<rMatrix3d> elem;
  find_sym_const_tensor(&elem, pointgroup);
  Array<rMatrix3d> aelem;
  LinkedList_to_Array(&aelem,elem);
  cout << "elem" << endl;
  cout.setf(ios::fixed);
  cout.precision(6);
  cout << aelem << endl;

  Structure str;
  ifstream strfile(strfilename);
  parse_structure_file(&str.cell,&str.atom_pos,&str.atom_type,atom_label,strfile);
  fix_atom_type(&str,lat,site_type_list,0);
  Array<Real> conc;
  calc_concentration(&conc, lat,site_type_list,str);
  cout << "conc" << endl << conc << endl;

  Array2d<Real> m;
  Array<Real> v;
  calc_noredtored(&m,&v,lat,site_type_list);
  cout << "noredtored_m" << endl << m << endl << "noredtored_v" << endl << v << endl;
  Array2d<Real> mi;
  calc_redtonored(&mi, m,v);
  cout << "redtonored" << endl << mi << endl;
}
